/**
 * @file asio.c - defines implementation for an
 * asynchronous serial IO driver for Propeller C bringup.
 * This module can be used with 5MHz or 10MHz crystal.
 * See header file for more details.
 */
#include "asio.h"

#undef ASIO_RXDEBUG

/**
 * Show bits on oscilloscope. This is for debug only ... bits come out LSB first
 * @param val is value to show
 * @param mask is the byte/word/long mask
 * @param pinmap is the pin bit map for bit stream output
 * @param syncmap is the bit map for bit stream oscope sync
 * @returns non-zero for looping;
 */
int show_val (int val, int mask, int pinmap, int syncmap)
{
    int ii = 1;
    // set direction then set word syncmap bit
    asm("or DIRA,%pinmap");
    asm("or DIRA,%syncmap");
    asm("or OUTA,%syncmap"); 

    while(ii & mask) { // this works
        if(val & ii) {
            asm("or   OUTA,%pinmap");
        }
        else {
            asm("andn OUTA,%pinmap");
            asm("andn OUTA,%pinmap"); // waveform symetry
            asm("andn OUTA,%pinmap"); // waveform symetry
        }
        ii = ii << 1;
    }
    asm("andn OUTA,%syncmap");
    return 1;
}

/**
 * Init direction bits.
 * Would init baud here, but using const vs var has different timing.
 */
void    asio_init (void)
{
    int bits = ASIO_TX_PIN_MAP;
    asm("MOV  DIRA,%bits");
}

/**
 * Sets a pinmap bit to a value for baudrate bit period.
 * @param val is bit value to set.
 * @param pin is pin bitmap
 */
static void setbit(int val, int pin)
{
    int per  = ASIO_BASE_PERIOD*ASIO_BAUDX_TX;
	int gttime;
    asm("mov        %gttime, cnt");
    asm("add        %gttime, %per");
    if(val) {
        asm("or     OUTA, %pin");
    }
    else {
        asm("andn   OUTA, %pin");
    }
    asm("waitcnt    %gttime, #0");
}


/**
 * Transmit a char on interface.
 * @param val is byte to send
 */
void    asio_tx (char val)
{
    int ii = 1;
    int mask = 0xFF;
    int pin  = ASIO_TX_PIN_MAP;
    int stp  = 1;

    if (ASIO_TX_INVERTED) {
        val = (volatile)~val;
        stp = (volatile)~stp & 1;
    }
    else { // waste time for symetry ... volatile to keep out optimization?
        val = (volatile)val;
        stp = (volatile)stp & 1;
    }

    for(ii = 0; ii < 4; ii++) {
        setbit(stp, pin); // stop bits
    }
        
    setbit((~stp & 1), pin); // start bit
    ii = 1;
    while(ii & mask) { // this works
        if(val & ii) {
            setbit(1, pin);
        }
        else {
            setbit(0, pin);
        }
        ii = ii << 1;
    }
    for(ii = 0; ii < ASIO_STOP_BITS; ii++) {
        setbit(stp, pin); // stop bits
    }
}

/**
 * Transmit a string on interface.
 * @param str is string to send.
 */
void    asio_txString (char* str)
{
    while(*str != 0) {
        asio_tx(*str);
        str++;
    }
}

/**
 * Get a bit from pinmap on the baud schedule
 * @param pin is pin bitmap
 * @returns int representing bit value 1 or 0
 */
static int getbit (int pin)
{
    int rc = 0;
    int bits;
    int per  = ASIO_BASE_PERIOD*ASIO_BAUDX_RX;
	int	grtime;
    asm("mov        %grtime, cnt");
    asm("add        %grtime, %per");
    asm("waitcnt    %grtime, %per");
    // for debugging
    //asm("xor  OUTA,#$1");
    
    // get bit
    asm("mov %bits, INA"); 
    rc = (bits & pin) ? ~ASIO_RX_INVERTED : ASIO_RX_INVERTED;
    rc &= 1;
    return rc;
}

/**
 * Get a byte from the serial interface.
 * param are set by # define constants in asio.h
 * @returns int representing byte received.
 */
int     asio_rx (void)
{
    int ii;
    int bits;
    int rx; 
    int byte = 0;
    int grtime;
    int per  = (ASIO_BASE_PERIOD*ASIO_BAUDX_RX);
    per += 800; // add some start sample margin for higher baudrates
     
#ifdef ASIO_RXDEBUG  
    // for debugging  
    asm("or   DIRA,#$7");
#endif

    do {
        asm("mov %bits, INA");
        rx = (bits & ASIO_RX_PIN_MAP) ? 0 : 1;
    } while(rx == ASIO_RX_INVERTED);

#ifdef ASIO_RXDEBUG  
    // for debugging  
    asm("xor   OUTA,#$4"); 
#endif

    // wait for start bit to pass    
    asm("mov        %grtime, cnt");
    asm("add        %grtime, %per");
    asm("waitcnt    %grtime, %per");

    // get bits on baud rate schedule
    for(ii = 0; ii < 8; ii++) {
        rx = getbit(ASIO_RX_PIN_MAP);
        byte = byte | (rx << 8);
        byte = byte >> 1;
    }
    getbit(ASIO_RX_PIN_MAP); // get stop bit
  
    return byte;
}

/**
 * Prefer using a buffer head/tail strategy provided by starting a COG.
 * For now use asio_rx byte by byte blocking calls.
 * Use calls to asio_rx until a terminator like "\r" is found.
 * All strings returned are null terminated.
 * @param str is a predefined input buffer of len.
 * @param len is maximum number of chars to receive in buffer.
 * @returns number of bytes received.
 */
int asio_rxString (char* str, int len)
{
    int byte = 0;
    int ii = 0;
    do {
        byte = asio_rx();
        str[ii] = byte;
        if(byte == ASIO_ENTER) {
           break;
        }
        ii++;
    } while(ii < len);
    
    if(ii < (len-1)) ii++;
    str[ii] = '\0';
    
    return ii;
}

//#define ASIO_TEST
#ifdef  ASIO_TEST
int main (int argc, char** argv)
{
    int ii = 0;
    char rxbuf[ASIO_BUFF_LEN+1]; 

    asio_init();
    
    asio_rx(); // block until user opens connection
    asio_txString("Hello World!\r\n");
    
    while(0) {
        //show_val(ii, 0xffff, 0x1, 0x2);
    }
    while(0) {
        asio_tx('U');
    }
    while(0) {
        asio_tx('H');
        asio_tx('i');
        asio_tx(' ');
    }
    while(0) {
        asio_txString("Hello World!\r\n");
    }
    while(0) {
        ii = asio_rx();
        //show_val(ii, 0xff, 0x1, 0x2);
    }
    while(0) {  // echo test
        asio_tx(asio_rx());
    }
    while(1) {  // echo string test
        ii = asio_rxString(rxbuf, ASIO_BUFF_LEN);
        //show_val(ii, 0xff, 0x1, 0x2);
        for(ii = 0; ii < 500; ii++) ; // pause before send
        asio_txString(rxbuf);
    }

    return 0;         
}
#endif
//ASIO_TEST
