/**
 * @file asio.h - Defines interfaces for an
 * asynchronous serial IO driver for Propeller C bringup.
 *
 * - This code works with ICC April Beta0 pre-release update
 * and April Alpha update, but not the March Alpha release.
 * Baudrate is defined by supplying constants to the 
 * implementations internal setbit() and getbit() functions
 * (this will be changed later to init parameters).
 *
 * - The waitcnt assembly command dominates delays, though
 * at higher baudrates, software delays contribute. The
 * code is tested with protoboard having 5 & 10 MHz crystals.
 * The tx and txString functions work with bauds at
 * 57600, 38400, 19200, 9600 ... other bauds may also work.
 *
 * - Receive functions are available, but most likely
 * less useful in their current form as buffering does
 * not seem possible without a way to set up a cog to
 * do the job in the background. A slow baud rate may help.
 * (now that a mechanism is available in beta for new cog,
 * priority will be given to creating a version with queues).
 *
 * - RS232 protocol parameters are {baud,N,1,8} ... stop bits
 * can be set in the hundreds to allow comm with slow
 * devices. This can be configured in the header file.
 *
 * - Public APIs have been tested with propeller pins 0/1
 * connected to a basic stamp module B2SX as well as
 * propeller pins 30/31 with hyperterm and other programs.
 *
 * - Tested on Propeller using 5MHz and 10MHz crystals.
 * The default project setting is 5MHz. To use with 10MHz,
 * in ICCPROP IDE, click Project->Options->Target and
 * select Clock Settings 80MHz: XTAL1+PLL8X 10MHz.
 *
 * - The actual bitrate of the rx/tx interface this module
 * provides is heavily dependent upon the underlying
 * compiler code generation. The waitcnt assembly instruction
 * is being used to mitigate this problem, but can not
 * eleminate the dependency entirely. Some improvements
 * can be made with this going forward, so it may be useful
 * to check for a driver update periodically.
 * 
 */
#ifndef __ASIO_H__
#define __ASIO_H__


/**
 * Define TX pin map. This code works with ICC April Alpha update.
 * Early ICC March 29 Alpha release has issues with asm(...) parameters.
 * Hard-coding the setbit asm call will make this work. i.e. asm("or  OUTA, $40000000");
 */
#define ASIO_TX_PIN_MAP (1 << 30)
/**
 * Define RX pin map. Using bit 31 for rx works fine.
 */
#define ASIO_RX_PIN_MAP (1 << 31)

/**
 * Use large number (100+) of TX stop bits for slow interfaces adjust at your liesure
 * Flow control not implemented - use 3 wire interface (rx,tx,gnd).
 */
#define ASIO_STOP_BITS  8      // 2 - N

/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_9600   56   
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_19200  23   
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_38400  6 
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_57600  1  

/**
 * Define for TX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_TX_9600   60   
/**
 * Define for TX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_TX_19200  28    
/**
 * Define for TX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_TX_38400  11    
/**
 * Define for TX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_TX_57600  5  

/**
 * Use these defines for software + waitcount RX/TX delays.
 * The use of waitcnt instruction asm("waitcnt %time, %period")
 * dominates timing at lower baudrates; software delays are
 * a factor in higher baudrates, but are mostly mitigated.
 */ 
#define ASIO_BAUDX_RX   ASIO_BAUDX_RX_57600    
#define ASIO_BAUDX_TX   ASIO_BAUDX_TX_57600    

/**
 * Base period is used with ASIO_BAUDX_dX for bit period calculations.
 */
#define ASIO_BASE_PERIOD    120

/**
 * Use this define to invert the interface sense (or not)
 */
#define ASIO_RT_INVERTED    0
/**
 * Use this define to invert the TX interface sense (or not)
 */
#define ASIO_TX_INVERTED    ASIO_RT_INVERTED
/**
 * Use this define to invert the RX interface sense (or not)
 */
#define ASIO_RX_INVERTED    ASIO_RT_INVERTED

#define ASIO_ENTER '\r'
#define ASIO_BUFF_LEN   0xFF     // some day this can serve as a buffer head/tail mask


/*
 * API functions. See implementation for usage.
 */
int     show_val(int val, int mask, int pinmap, int syncmap);
void    asio_init(void);
void    asio_tx(char c);
void    asio_txString(char* str);
int     asio_rx(void);
int     asio_rxString(char* str, int len);
 
#endif
//__ASIO_H__


